/* BdfServer - Copyright (c) 2006-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.users;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.exportation.table.TableExportParameters;
import fr.exemole.bdfserver.api.managers.PermissionManager;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.api.users.BdfUserConstants;
import fr.exemole.bdfserver.api.users.BdfUserPrefs;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.FicheTableParametersBuilder;
import fr.exemole.bdfserver.tools.exportation.table.TableExportParametersBuilder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.SimpleTimeZone;
import net.fichotheque.EditOrigin;
import net.fichotheque.Subset;
import net.fichotheque.corpus.fiche.ContentChecker;
import net.fichotheque.exportation.table.TableExportConstants;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.utils.TableExportUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.text.TypoOptions;
import net.mapeadores.util.text.ValidExtension;
import net.mapeadores.opendocument.io.SheetWriter;


/**
 *
 * @author Vincent Calame
 */
public final class BdfUserUtils {

    private final static DateFormat ISOTIME_DATEFORMAT = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

    static {
        ISOTIME_DATEFORMAT.setTimeZone(new SimpleTimeZone(0, "GMT"));
    }

    private BdfUserUtils() {
    }

    public static ContentChecker getContentChecker(BdfServer bdfServer, BdfUser bdfUser) {
        return bdfServer.getFichothequeEditorProvider().getContentChecker();
    }

    public static boolean isAdmin(BdfServer bdfServer, BdfUser bdfUser) {
        return bdfServer.getPermissionManager().isAdmin(bdfUser.getRedacteur());
    }

    public static boolean isSame(BdfUser bdfUser, Redacteur redacteur) {
        return bdfUser.getRedacteur().getGlobalId().equals(redacteur.getGlobalId());
    }

    public static boolean hasRole(PermissionManager permissionManager, BdfUser bdfUser, Role role) {
        return hasRole(permissionManager, bdfUser.getRedacteur(), role.getName());
    }

    public static boolean hasRole(PermissionManager permissionManager, Redacteur redacteur, String roleName) {
        for (Role role : permissionManager.getRoleList(redacteur)) {
            if (role.getName().equals(roleName)) {
                return true;
            }
        }
        return false;
    }

    public static BdfUser getFirstAdminBdfUser(BdfServer bdfServer) {
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        for (Sphere sphere : bdfServer.getFichotheque().getSphereList()) {
            for (Redacteur redacteur : sphere.getRedacteurList()) {
                if (permissionManager.isAdmin(redacteur)) {
                    return bdfServer.createBdfUser(redacteur);
                }
            }
        }
        throw new IllegalStateException("No redacteur admin");
    }

    public static MessageLocalisation getMessageLocalisation(BdfServer bdfServer, BdfUser bdfUser) {
        return bdfServer.getL10nManager().getMessageLocalisation(bdfUser.getLangPreference());
    }

    public static EditOrigin newEditOrigin(String source) {
        String isoTime;
        synchronized (ISOTIME_DATEFORMAT) {
            isoTime = ISOTIME_DATEFORMAT.format(new Date());
        }
        return new InternalEditOrigin(null, null, isoTime, source);
    }

    public static EditOrigin newEditOrigin(String sessionId, String redacteurGlobalId, String source) {
        String isoTime;
        synchronized (ISOTIME_DATEFORMAT) {
            isoTime = ISOTIME_DATEFORMAT.format(new Date());
        }
        return new InternalEditOrigin(sessionId, redacteurGlobalId, isoTime, source);
    }

    public static String getSimpleTemplateName(BdfUser bdfUser, TransformationKey transformationKey) {
        AttributeKey attributeKey = BdfUserSpace.toSimpleTemplateAttributeKey(transformationKey);
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        if (attribute == null) {
            return TemplateKey.DEFAULT_NAME;
        }
        return attribute.getFirstValue();
    }

    public static TemplateKey getSimpleTemplateKey(BdfUser bdfUser, TransformationKey transformationKey) {
        AttributeKey attributeKey = BdfUserSpace.toSimpleTemplateAttributeKey(transformationKey);
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        TemplateKey templateKey = null;
        if (attribute != null) {
            try {
                templateKey = TemplateKey.parse(transformationKey, attribute.getFirstValue());
            } catch (ParseException pe) {

            }
        }
        if (templateKey == null) {
            templateKey = TemplateKey.toDefault(transformationKey);
        }
        return templateKey;
    }

    public static String getStreamTemplateName(BdfUser bdfUser, TransformationKey transformationKey, String extension) {
        AttributeKey attributeKey = BdfUserSpace.toStreamTemplateAttributeKey(transformationKey, extension);
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        if (attribute == null) {
            return TemplateKey.DEFAULT_NAME;
        }
        return attribute.getFirstValue();
    }

    public static TemplateKey getStreamTemplateKey(BdfUser bdfUser, TransformationKey transformationKey, ValidExtension validExtension) {
        AttributeKey attributeKey = BdfUserSpace.toStreamTemplateAttributeKey(transformationKey, validExtension.toString());
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        TemplateKey templateKey = null;
        if (attribute != null) {
            try {
                templateKey = TemplateKey.parse(transformationKey, validExtension, attribute.getFirstValue());
            } catch (ParseException pe) {

            }
        }
        if (templateKey == null) {
            templateKey = TemplateKey.toDefault(transformationKey, validExtension);
        }
        return templateKey;
    }

    public static String getSubsetTableExportName(BdfUser bdfUser, Subset subset) {
        AttributeKey attributeKey = BdfUserSpace.toTableAttributeKey(subset);
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        if (attribute == null) {
            return null;
        }
        return attribute.getFirstValue();
    }

    public static String getRootUrl(@Nullable BdfUser bdfUser) {
        if (bdfUser != null) {
            String sessionUrl = (String) bdfUser.getParameterValue(BdfUserConstants.SESSION_ROOTURL);
            if (sessionUrl != null) {
                return sessionUrl;
            }
        }
        return "";
    }

    public static boolean isWithJavascript(@Nullable BdfUser bdfUser) {
        if (bdfUser != null) {
            Boolean bool = (Boolean) bdfUser.getParameterValue(BdfUserConstants.HTML_WITHJAVASCRIPT);
            if (bool != null) {
                return bool;
            }
        }
        return false;
    }

    public static TypoOptions getTypoOptions(@Nullable BdfUser bdfUser) {
        if (bdfUser != null) {
            TypoOptions typoOptions = (TypoOptions) bdfUser.getParameterValue(BdfUserConstants.FORM_TYPOOPTIONS);
            if (typoOptions != null) {
                return typoOptions;
            }
        }
        return TypoOptions.getTypoOptions(Locale.FRENCH);
    }

    public static FicheTableParameters getDefaultFicheTableParameters(BdfUser bdfUser) {
        BdfUserPrefs prefs = bdfUser.getPrefs();
        FicheTableParametersBuilder builder = new FicheTableParametersBuilder();
        Attribute patternModeAttribute = prefs.getAttributes().getAttribute(BdfUserSpace.TABLEEXPORT_PATTERNMODE_KEY);
        if (patternModeAttribute != null) {
            try {
                builder.setPatternMode(BdfTableExportUtils.patternModeToShort(patternModeAttribute.getFirstValue()));
            } catch (IllegalArgumentException iae) {

            }
        }
        Attribute withAttribute = prefs.getAttributes().getAttribute(BdfUserSpace.TABLEEXPORT_WITH_KEY);
        if (withAttribute != null) {
            int valueLength = withAttribute.size();
            for (int i = 0; i < valueLength; i++) {
                builder.putWith(withAttribute.get(i));
            }
        }
        return builder.toFicheTableParameters();
    }

    public static TableExportParameters getDefaultTableExportParameters(BdfUser bdfUser) {
        TableExportParametersBuilder builder = new TableExportParametersBuilder();
        builder.setHeaderType(getDefaultHeaderType(bdfUser));
        builder.setFicheTableParameters(getDefaultFicheTableParameters(bdfUser));
        return builder.toTableExportParameters();
    }

    public static short getDefaultHeaderType(BdfUser bdfUser) {
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(BdfUserSpace.TABLEEXPORT_HEADERTYPE_KEY);
        if (attribute != null) {
            try {
                return TableExportUtils.headerTypeToShort(attribute.getFirstValue());
            } catch (IllegalArgumentException iae) {

            }
        }
        return TableExportConstants.COLUMNTITLE_HEADER;
    }

    public static boolean getBoolean(BdfUser bdfUser, AttributeKey attributeKey) {
        Attribute attribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        if (attribute == null) {
            return false;
        }
        String firstValue = attribute.getFirstValue().toLowerCase();
        switch (firstValue) {
            case "0":
            case "false":
                return false;
            default:
                return true;
        }

    }

    public static SheetWriter getSupplementarySheetWriter(BdfServer bdfServer, BdfUser bdfUser) {
        if (!getBoolean(bdfUser, BdfUserSpace.TABLEEXPORT_STATUSSHEET_KEY)) {
            return null;
        }
        return new StatusSheetWriter(bdfServer, bdfUser);
    }


    private static class InternalEditOrigin implements EditOrigin {

        private final String redacteurGlobalId;
        private final String isoTime;
        private final String source;
        private final String sessionId;

        private InternalEditOrigin(String sessionId, String redacteurGlobalId, String isoTime, String source) {
            this.sessionId = sessionId;
            this.redacteurGlobalId = redacteurGlobalId;
            this.isoTime = isoTime;
            this.source = source;
        }

        @Override
        public String getSessionId() {
            return sessionId;
        }

        @Override
        public String getRedacteurGlobalId() {
            return redacteurGlobalId;
        }

        @Override
        public String getISOTime() {
            return isoTime;
        }

        @Override
        public String getSource() {
            return source;
        }

    }


}
