/* BdfServer - Copyright (c) 2008-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xml.ui;

import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import fr.exemole.bdfserver.tools.ui.components.UiComponentUtils;
import java.io.IOException;
import java.util.List;
import net.fichotheque.include.ExtendedIncludeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.Labels;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class UiComponentsXMLPart extends XMLPart {

    public UiComponentsXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendUiComponents(UiComponents uiComponents) throws IOException {
        openTag("ui-components");
        openTag("metadata");
        appendCommentDef(uiComponents);
        closeTag("metadata");
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (uiComponent instanceof CommentUi) {
                CommentUi commentUi = (CommentUi) uiComponent;
                startOpenTag("comment");
                addAttribute("name", commentUi.getCommentName());
                closeEmptyTag();
            } else if (uiComponent instanceof FieldUi) {
                appendFieldUi((FieldUi) uiComponent);
            } else if (uiComponent instanceof IncludeUi) {
                IncludeUi includeUi = (IncludeUi) uiComponent;
                ExtendedIncludeKey extendedIncludeKey = includeUi.getExtendedIncludeKey();
                if (extendedIncludeKey == null) {
                    appendSpecialIncludeUi(includeUi);
                } else {
                    appendSubsetIncludeUi(extendedIncludeKey, includeUi);
                }
            }
        }
        closeTag("ui-components");
    }

    private void appendCommentDef(UiComponents uiComponents) throws IOException {
        List<CommentUi> list = UiComponentUtils.getCommentUiList(uiComponents);
        for (CommentUi commentUi : list) {
            startOpenTag("comment-def");
            addAttribute("name", commentUi.getCommentName());
            addAttribute("location", UiUtils.maskToString(commentUi.getLocation()));
            endOpenTag();
            for (Lang lang : commentUi.getLangs()) {
                startOpenTag("html");
                addAttribute("xml:lang", lang.toString());
                endOpenTag();
                addCData(commentUi.getHtmlByLang(lang));
                closeTag("html", false);
            }
            appendAttributes(commentUi);
            closeTag("comment-def");
        }
    }

    private void appendFieldUi(FieldUi fieldUi) throws IOException {
        startOpenTag("field-ui");
        addAttribute("field-key", fieldUi.getName());
        endOpenTag();
        appendInputOption(fieldUi.getInputOption());
        appendValueOption(fieldUi.getValueOption());
        appendAttributes(fieldUi);
        closeTag("field-ui");
    }

    private void appendSpecialIncludeUi(IncludeUi includeUi) throws IOException {
        startOpenTag("special-ui");
        addAttribute("include-key", includeUi.getName());
        endOpenTag();
        appendCustomLabels(includeUi);
        appendLimitOption(includeUi.getLimitOption());
        appendValueOption(includeUi.getValueOption());
        appendAttributes(includeUi);
        closeTag("special-ui");
    }

    private void appendSubsetIncludeUi(ExtendedIncludeKey includeKey, IncludeUi includeUi) throws IOException {
        openTag("subset-ui");
        if (includeKey.isMaster()) {
            addEmptyElement("master");
        }
        addSimpleElement("subset", includeKey.getSubsetKey().toString());
        String mode = includeKey.getMode();
        if (mode.length() > 0) {
            addSimpleElement("mode", mode);
        }
        int poidsFilter = includeKey.getPoidsFilter();
        if (poidsFilter != -1) {
            addSimpleElement("poids", String.valueOf(poidsFilter));
        }
        appendCustomLabels(includeUi);
        appendInputOption(includeUi.getInputOption());
        appendValueOption(includeUi.getValueOption());
        appendAttributes(includeUi);
        closeTag("subset-ui");
    }

    private void appendInputOption(InputOption inputOption) throws IOException {
        if (inputOption == null) {
            return;
        }
        startOpenTag("input");
        addAttribute("type", inputOption.getInputType());
        addAttribute("width", inputOption.getWidthType());
        addAttribute("rows", inputOption.getRows());
        closeEmptyTag();
    }

    private void appendLimitOption(LimitOption limitOption) throws IOException {
        if (limitOption == null) {
            return;
        }
        startOpenTag("limit");
        addAttribute("nonew", (limitOption.isNoNewAllowed()) ? "1" : "0");
        closeEmptyTag();
    }

    private void appendCustomLabels(IncludeUi includeUi) throws IOException {
        Labels labels = includeUi.getCustomLabels();
        if (labels == null) {
            return;
        }
        openTag("custom");
        LabelUtils.addLabels(this, labels);
        closeTag("custom");
    }

    private void appendValueOption(ValueOption valueOption) throws IOException {
        if (valueOption == null) {
            return;
        }
        boolean mandatory = valueOption.isMandatory();
        String defaultValue = valueOption.getDefaultValue();
        boolean withDefaultValue = (!(defaultValue.isEmpty()));
        if ((mandatory) || (withDefaultValue)) {
            startOpenTag("value");
            if (mandatory) {
                addAttribute("mandatory", "1");
            }
            if (withDefaultValue) {
                addAttribute("default", defaultValue);
            }
            closeEmptyTag();
        }
    }

    private void appendAttributes(UiComponent uiComponent) throws IOException {
        AttributeUtils.addAttributes(this, uiComponent.getAttributes());
    }

}
