/* BdfServer_DirectoryStorage - Copyright (c) 2006-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.implementations;

import fr.exemole.bdfserver.api.storage.BalayageStorage;
import fr.exemole.bdfserver.api.storage.StorageContent;
import fr.exemole.bdfserver.storage.directory.bdfdata.StorageFile;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.EditOrigin;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class BalayageStorageImpl implements BalayageStorage {

    private final File varBalayageDirectory;
    private final File backupBalayageDirectory;

    public BalayageStorageImpl(File balayagesDirectory, File backupBalayageDirectory) {
        this.varBalayageDirectory = balayagesDirectory;
        this.backupBalayageDirectory = backupBalayageDirectory;
    }

    @Override
    public BalayageStorage.Unit[] checkStorage() {
        SortedMap<String, BalayageStorage.Unit> map = new TreeMap<String, BalayageStorage.Unit>();
        if ((varBalayageDirectory.exists()) && (varBalayageDirectory.isDirectory())) {
            for (File f : varBalayageDirectory.listFiles()) {
                if (f.isDirectory()) {
                    String name = f.getName();
                    if (testName(name)) {
                        map.put(name, buildUnit(name, f));
                    }
                }
            }
        }
        return map.values().toArray(new Unit[map.size()]);
    }

    @Override
    public Unit getBalayageStorageUnit(String balayageName) {
        if (!testName(balayageName)) {
            return null;
        }
        File varDir = new File(varBalayageDirectory, balayageName);
        if (varDir.exists()) {
            if (!varDir.isDirectory()) {
                return null;
            }
            return buildUnit(balayageName, varDir);
        }
        return null;
    }

    @Override
    public StorageContent getStorageContent(String balayageName, String contentPath) {
        File balayageDir = new File(varBalayageDirectory, balayageName);
        if ((!balayageDir.exists()) || (!balayageDir.isDirectory())) {
            return null;
        }
        contentPath = checkPath(contentPath);
        if (contentPath == null) {
            return null;
        }
        StorageFile storageFile = getStorageFile(balayageName, contentPath);
        if (!storageFile.exists()) {
            return null;
        }
        return StorageUtils.toStorageContent(contentPath, storageFile.getFile());
    }

    @Override
    public void saveStorageContent(String balayageName, String contentPath, InputStream inputStream, EditOrigin editOrigin) throws IOException {
        if (!testName(balayageName)) {
            throw new IllegalArgumentException(balayageName + " is not a technical name");
        }
        contentPath = checkPath(contentPath);
        if (contentPath == null) {
            throw new IllegalArgumentException(contentPath + " is not a relative path");
        }
        StorageFile storageFile = getStorageFile(balayageName, contentPath);
        try (OutputStream os = storageFile.archiveAndGetOutputStream(editOrigin)) {
            IOUtils.copy(inputStream, os);
        }
    }

    @Override
    public boolean removeStorageContent(String balayageName, String contentPath, EditOrigin editOrigin) {
        if (!testName(balayageName)) {
            return false;
        }
        contentPath = checkPath(contentPath);
        if (contentPath == null) {
            return false;
        }
        StorageFile storageFile = getStorageFile(balayageName, contentPath);
        if (!storageFile.exists()) {
            return false;
        }
        storageFile.archiveAndDelete(editOrigin);
        return true;
    }

    @Override
    public boolean removeBalayage(String balayageName, EditOrigin editOrigin) {
        if (!testName(balayageName)) {
            return false;
        }
        File rootDirectory = new File(varBalayageDirectory, balayageName);
        if (!rootDirectory.exists()) {
            return false;
        }
        File rootBackupDirectory = null;
        if (backupBalayageDirectory != null) {
            rootBackupDirectory = new File(backupBalayageDirectory, balayageName);
        }
        StorageFile.archiveAndDeleteDirectory(rootDirectory, rootBackupDirectory, editOrigin);
        return true;
    }

    private DirectoryBalayageStorageUnit buildUnit(String name, File balayageDir) {
        List<StorageContent> storageContentList = new ArrayList<StorageContent>();
        for (File file : balayageDir.listFiles()) {
            String fileName = file.getName();
            if (fileName.equals("_def.xml")) {
            } else if (file.isDirectory()) {
                StorageUtils.populate(storageContentList, file, "");
            } else {
                storageContentList.add(StorageUtils.toStorageContent(file.getName(), file));
            }
        }
        List<StorageContent> finalStorageContentList = StorageUtils.wrap(storageContentList.toArray(new StorageContent[storageContentList.size()]));
        return new DirectoryBalayageStorageUnit(name, balayageDir, finalStorageContentList);
    }

    private StorageFile getStorageFile(String balayageName, String contentPath) {
        String path = balayageName + File.separator + contentPath;
        return StorageFile.build(varBalayageDirectory, backupBalayageDirectory, path);
    }


    private static class DirectoryBalayageStorageUnit implements Unit {

        private final String name;
        private final File directory;
        private final List<StorageContent> storageContentList;

        private DirectoryBalayageStorageUnit(String name, File directory, List<StorageContent> storageContentList) {
            this.name = name;
            this.directory = directory;
            this.storageContentList = storageContentList;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public StorageContent getStorageContent(String path) {
            path = StringUtils.normalizeRelativePath(path);
            if (path == null) {
                return null;
            }
            File file = new File(directory, path);
            if ((file.exists()) && (!file.isDirectory())) {
                return StorageUtils.toStorageContent(path, file);
            } else {
                return null;
            }
        }

        @Override
        public List<StorageContent> getStorageContentList() {
            return storageContentList;
        }

    }

    private static boolean testName(String s) {
        return StringUtils.isTechnicalName(s, true);
    }

    private static String checkPath(String path) {
        path = StringUtils.normalizeRelativePath(path);
        if (path == null) {
            return null;
        }
        return path;
    }

}
