/* BdfServer_Html - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.ficheform.GeopointProprieteSubfieldsElement;
import fr.exemole.bdfserver.api.ficheform.HiddenFieldElement;
import fr.exemole.bdfserver.api.ficheform.ImageProprieteSubfieldsElement;
import fr.exemole.bdfserver.api.ficheform.ItemFieldElement;
import fr.exemole.bdfserver.api.ficheform.LangFieldElement;
import fr.exemole.bdfserver.api.ficheform.MontantInformationSubfieldsElement;
import fr.exemole.bdfserver.api.ficheform.MontantProprieteSubfieldsElement;
import fr.exemole.bdfserver.api.ficheform.PersonneProprieteSubfieldsElement;
import fr.exemole.bdfserver.api.ficheform.TextFieldElement;
import fr.exemole.bdfserver.html.consumers.FicheForm;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.SubfieldKey;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.TrustedHtml;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.models.PersonCore;
import net.mapeadores.util.money.Currencies;
import net.mapeadores.util.money.ExtendedCurrency;


/**
 * Série de méthodes statiques traduisant en code HTML des instances de
 * FieldElement.
 *
 * @author Vincent Calame
 */
public final class CorpusFieldFormHtml {

    private CorpusFieldFormHtml() {
    }

    public static boolean printComment(HtmlPrinter hp, TrustedHtml html) {
        hp
                .DIV("ficheform-Comment")
                .__breakLine()
                .__append(html)
                .__breakLine()
                ._DIV();
        return true;
    }

    public static boolean printImageProprieteSubfields(HtmlPrinter hp, ImageProprieteSubfieldsElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        String name = getPrefixedName(corpusField, namePrefix);
        String src = formElement.getSrc();
        String srcName = getPrefixedName(fieldKey, SubfieldKey.SRC_SUBTYPE, namePrefix);
        String altString = formElement.getAlt();
        String altName = getPrefixedName(fieldKey, SubfieldKey.ALT_SUBTYPE, namePrefix);
        String titleString = formElement.getTitle();
        String titleName = getPrefixedName(fieldKey, SubfieldKey.TITLE_SUBTYPE, namePrefix);
        String genId = hp.generateId();
        String srcId = genId + "_src";
        String altId = genId + "_alt";
        String titleId = genId + "_title";
        hp
                .SECTION(toEntryAttributes(name, formElement))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, srcId))
                .__(FicheForm.LARGE_CELL, () -> {
                    hp
                            .TABLE("ficheform-SubfieldsTable")
                            .__(startSubFieldRow(hp, "_ label.edition.src", srcId))
                            .INPUT_text(HA.name(srcName).id(srcId).value(src).size("45"))
                            .__(endSubfieldRow(hp))
                            .__(startSubFieldRow(hp, "_ label.edition.alt", altId))
                            .INPUT_text(HA.name(altName).id(altId).value(altString).size("45"))
                            .__(endSubfieldRow(hp))
                            .__(startSubFieldRow(hp, "_ label.edition.title", titleId))
                            .INPUT_text(HA.name(titleName).id(titleId).value(titleString).size("45"))
                            .__(endSubfieldRow(hp))
                            ._TABLE();
                })
                ._SECTION();
        return true;
    }

    public static boolean printLangField(HtmlPrinter hp, LangFieldElement formElement, String namePrefix, Lang lang) {
        if (formElement.getAvailableLangArray() != null) {
            printSelectLangField(hp, formElement, namePrefix, lang);
            return false;
        }
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        String langId = hp.generateId();
        Lang currentLang = formElement.getLang();
        String langString;
        if (currentLang == null) {
            langString = "";
        } else {
            langString = currentLang.toString();
        }
        hp
                .SECTION(toEntryAttributes(name, formElement, "lang-code").attr("data-ficheform-limit", "1"))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, langId))
                .__(FicheForm.XSMALL_CELL, () -> {
                    hp
                            .INPUT_text(HA.name(name).id(langId).size("4").value(langString).classes("ficheform-Full").attr("data-ficheform-role", "textinput"));
                })
                ._SECTION();
        return true;
    }

    private static boolean printSelectLangField(HtmlPrinter hp, LangFieldElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        Lang currentLang = formElement.getLang();
        Lang[] array = formElement.getAvailableLangArray();
        int length = array.length;
        if (length == 0) {
            throw new ImplementationException("formElement.getAvailableLangArray().length == 0");
        } else if ((length == 1) && ((currentLang == null) || (currentLang.equals(array[0])))) {
            hp
                    .INPUT_hidden(name, array[0].toString());
            return false;
        }
        String langId = hp.generateId();
        hp
                .SECTION(toEntryAttributes(name, formElement))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, langId))
                .__(FicheForm.XSMALL_CELL, () -> {
                    hp
                            .SELECT(HA.name(name).id(langId));
                    boolean here = false;
                    for (int i = 0; i < length; i++) {
                        Lang availableLang = array[i];
                        String currLangString = availableLang.toString();
                        boolean selected = false;
                        if (currentLang != null) {
                            selected = availableLang.equals(currentLang);
                        } else if (i == 0) {
                            selected = true;
                        }
                        if (selected) {
                            here = true;
                        }
                        hp
                                .OPTION(currLangString, selected)
                                .__escape(currLangString)
                                .__dash()
                                .__localize(currLangString)
                                ._OPTION();
                    }
                    if (!here) {
                        String otherLangString = currentLang.toString();
                        hp
                                .OPTION(otherLangString, true)
                                .__escape(otherLangString)
                                .__dash()
                                .__localize(otherLangString)
                                ._OPTION();
                    }
                    hp
                            ._SELECT();
                })
                ._SECTION();
        return true;
    }

    public static boolean printHiddenField(HtmlPrinter hp, HiddenFieldElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        hp
                .DIV()
                .INPUT_hidden(name, formElement.getValue())
                ._DIV();
        return true;
    }

    public static boolean printItemField(HtmlPrinter hp, ItemFieldElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        String entryType = getEntryType(corpusField);
        String widthType = formElement.getWidthType();
        int defaultSize = CommonFormHtml.getDefaultSize(widthType);
        String param = null;
        SubsetKey sphereKey = formElement.getSphereKey();
        if (sphereKey != null) {
            param = sphereKey.toString();
        }
        int rows = formElement.getRows();
        HtmlAttributes inputAttr = hp.name(name).classes("ficheform-Full").attr("data-ficheform-role", "textinput");
        CommonFormHtml.addTextInputAttributes(inputAttr, formElement.getAttributes(), lang);
        hp
                .SECTION(toEntryAttributes(name, formElement, entryType)
                        .attr((entryType != null), "data-ficheform-limit", getPiocheLimit(corpusField))
                        .attr((param != null), "data-ficheform-param", param))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, inputAttr.id()))
                .__(FicheForm.cell(widthType), () -> {
                    if (rows == 1) {
                        hp
                                .INPUT_text(inputAttr.value(formElement.getValue()).size(String.valueOf(defaultSize)));
                    } else {
                        hp
                                .TEXTAREA(inputAttr.cols(defaultSize).rows(rows))
                                .__escape(formElement.getValue(), true)
                                ._TEXTAREA();
                    }
                })
                ._SECTION();
        return true;
    }

    public static boolean printMontantProprieteSubfields(HtmlPrinter hp, MontantProprieteSubfieldsElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        String currentCur = formElement.getCur();
        FieldKey fieldKey = corpusField.getFieldKey();
        String genId = hp.generateId();
        hp
                .SECTION(toEntryAttributes(name, formElement))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, genId))
                .__(FicheForm.LARGE_CELL, () -> {
                    if (formElement.isUnique()) {
                        hp
                                .EM()
                                .__escape(currentCur)
                                ._EM()
                                .INPUT_hidden(getPrefixedName(fieldKey, SubfieldKey.CURRENCY_SUBTYPE, namePrefix), currentCur);
                    } else {
                        boolean inSelection = false;
                        Currencies currencies = formElement.getCurrencies();
                        hp.
                                SELECT(getPrefixedName(fieldKey, SubfieldKey.CURRENCY_SUBTYPE, namePrefix));
                        for (ExtendedCurrency currency : currencies) {
                            String curString = currency.getCurrencyCode();
                            boolean selected = curString.equals(formElement.getCur());
                            if (selected) {
                                inSelection = true;
                            }
                            hp
                                    .OPTION(curString, selected)
                                    .__escape(curString)
                                    ._OPTION();
                        }
                        if (!inSelection) {
                            hp
                                    .OPTION(formElement.getCur(), true)
                                    .__escape('?')
                                    .__escape(formElement.getCur())
                                    .__escape('?')
                                    ._OPTION();
                        }
                        hp
                                ._SELECT();
                    }
                    hp
                            .__space()
                            .INPUT_text(HA.name(getPrefixedName(fieldKey, SubfieldKey.VALUE_SUBTYPE, namePrefix)).id(genId).value(formElement.getNum()).size("30"));
                })
                ._SECTION();
        return true;
    }

    public static boolean printMontantInformationSubfields(HtmlPrinter hp, MontantInformationSubfieldsElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        String name = getPrefixedName(corpusField, namePrefix);
        String genId = hp.generateId();
        List<MontantInformationSubfieldsElement.Entry> entryList = formElement.getEntryList();
        String firstId;
        if (entryList.size() > 0) {
            firstId = genId + "_" + entryList.get(0).getCurrency().getCurrencyCode();
        } else {
            firstId = genId + "_others";
        }
        hp
                .SECTION(toEntryAttributes(name, formElement))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, firstId))
                .__(FicheForm.LARGE_CELL, () -> {
                    hp
                            .TABLE("ficheform-SubfieldsTable");
                    for (MontantInformationSubfieldsElement.Entry entry : entryList) {
                        ExtendedCurrency currency = entry.getCurrency();
                        SubfieldKey subfieldKey = SubfieldKey.toAmountSubfieldKey(fieldKey, currency);
                        String subname = CorpusFieldFormHtml.getPrefixedName(subfieldKey.toString(), namePrefix);
                        String currencyId = genId + "_" + currency.getCurrencyCode();
                        hp
                                .__(startCurrencySubFieldRow(hp, currency, currencyId))
                                .INPUT_text(HA.name(subname).id(currencyId).value(entry.getMontantValue()).size("30"))
                                .__(endSubfieldRow(hp));
                    }
                    String wrong = formElement.getOthersValue();
                    if (wrong.length() > 0) {
                        String wrongName = getPrefixedName(fieldKey, SubfieldKey.OTHERS_SUBTYPE, namePrefix);
                        String othersId = genId + "_others";
                        hp
                                .__(startSubFieldRow(hp, "_ label.edition.others", othersId))
                                .TEXTAREA(HA.name(wrongName).id(othersId).rows(4).cols(45))
                                .__escape(wrong, true)
                                ._TEXTAREA()
                                .__(endSubfieldRow(hp));
                    }
                    hp
                            ._TABLE();
                })
                ._SECTION();
        return true;
    }

    public static boolean printGeopointProprieteSubfields(HtmlPrinter hp, GeopointProprieteSubfieldsElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        FieldKey fieldKey = corpusField.getFieldKey();
        String[] array = formElement.getAdresseFieldNameArray();
        String latName = getPrefixedName(fieldKey, SubfieldKey.LATITUDE_SUBTYPE, namePrefix);
        String lonName = getPrefixedName(fieldKey, SubfieldKey.LONGITUDE_SUBTYPE, namePrefix);
        String genId = hp.generateId();
        String latId = genId + "_lat";
        String lonId = genId + "_lon";
        String param = null;
        if (array != null) {
            StringBuilder buf = new StringBuilder();
            boolean next = false;
            for (String fieldName : array) {
                if (next) {
                    buf.append(",");
                } else {
                    next = true;
                }
                buf.append(getPrefixedName(fieldName, namePrefix));
            }
            param = buf.toString();
        }
        hp
                .SECTION(toEntryAttributes(name, formElement, "geocodage")
                        .attr((param != null), "data-ficheform-param", param))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, latId))
                .__(FicheForm.LARGE_CELL, () -> {
                    hp
                            .TABLE("ficheform-SubfieldsTable")
                            .__(startSubFieldRow(hp, "_ label.edition.latitude", latId))
                            .INPUT_text(HA.name(latName).id(latId).value(formElement.getLatitude()).size("15").attr("data-ficheform-role", "latitude"))
                            .__(endSubfieldRow(hp))
                            .__(startSubFieldRow(hp, "_ label.edition.longitude", lonId))
                            .INPUT_text(HA.name(lonName).id(lonId).value(formElement.getLongitude()).size("15").attr("data-ficheform-role", "longitude"))
                            .__(endSubfieldRow(hp))
                            ._TABLE();
                })
                ._SECTION();
        return true;
    }

    public static boolean printPersonneProprieteSubfields(HtmlPrinter hp, PersonneProprieteSubfieldsElement formElement, String namePrefix, Lang lang) {
        CorpusField corpusField = formElement.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        PersonCore personCore = formElement.getPersonCore();
        boolean withNonlatin = formElement.isWithNonlatin();
        if (personCore.getNonlatin().length() > 0) {
            withNonlatin = true;
        }
        HtmlAttributes inputAttr = HA.classes("ficheform-Full").size("30");
        String surnameParamName = CorpusFieldFormHtml.getPrefixedName(fieldKey, SubfieldKey.SURNAME_SUBTYPE, namePrefix);
        String forenameParamName = CorpusFieldFormHtml.getPrefixedName(fieldKey, SubfieldKey.FORENAME_SUBTYPE, namePrefix);
        String genId = hp.generateId();
        String surnameId = genId + "_surname";
        String forenameId = genId + "_forename";
        hp
                .SECTION(toEntryAttributes(surnameParamName, formElement))
                .__(printStandardLabel(hp, hp.getLocalization((withNonlatin) ? "_ label.sphere.surname_latin" : "_ label.sphere.surname"), surnameId))
                .__(FicheForm.MEDIUM_CELL, () -> {
                    hp
                            .INPUT_text(inputAttr.name(surnameParamName).id(surnameId).value(personCore.getSurname()));
                })
                ._SECTION();
        hp
                .SECTION(toEntryAttributes(forenameParamName, formElement))
                .__(printStandardLabel(hp, hp.getLocalization((withNonlatin) ? "_ label.sphere.forename_latin" : "_ label.sphere.forename"), forenameId))
                .__(FicheForm.MEDIUM_CELL, () -> {
                    hp
                            .INPUT_text(inputAttr.name(forenameParamName).id(forenameId).value(personCore.getForename()));
                });
        if ((!formElement.isWithoutSurnameFirst()) || (personCore.isSurnameFirst())) {
            hp
                    .__(FicheForm.LARGE_CELL, () -> {
                        String surnameFirstParamName = getPrefixedName(fieldKey, SubfieldKey.SURNAMEFIRST_SUBTYPE, namePrefix);
                        String surnameFirstId = genId + "_surnamefirst";
                        hp
                                .DIV("ficheform-Choice")
                                .INPUT_checkbox(HA.name(surnameFirstParamName).id(surnameFirstId).value("1").checked(personCore.isSurnameFirst()))
                                .LABEL_for(surnameFirstId)
                                .__localize("_ label.sphere.surnamefirst")
                                ._LABEL()
                                ._DIV();
                    });
        }
        hp
                ._SECTION();
        if (withNonlatin) {
            String nonlatinParamName = getPrefixedName(fieldKey, SubfieldKey.NONLATIN_SUBTYPE, namePrefix);
            String nonlatinId = genId + "_nonlatin";
            hp
                    .SECTION(toEntryAttributes(nonlatinParamName, formElement))
                    .__(printStandardLabel(hp, hp.getLocalization("_ label.sphere.nonlatin"), nonlatinId))
                    .__(FicheForm.MEDIUM_CELL, () -> {
                        hp
                                .INPUT_text(inputAttr.name(nonlatinParamName).id(nonlatinId).value(personCore.getNonlatin()));
                    })
                    ._SECTION();
        }
        return true;
    }

    public static boolean printTextField(HtmlPrinter hp, TextFieldElement formElement, String namePrefix, Lang lang) {
        String textId = hp.generateId();
        CorpusField corpusField = formElement.getCorpusField();
        String name = getPrefixedName(corpusField, namePrefix);
        hp
                .SECTION(toEntryAttributes(name, formElement, "section", "ficheform-section-Entry"))
                .__breakLine();
        hp
                .DIV("ficheform-section-Label")
                .LABEL(CommonFormHtml.addTooltip(HA.forId(textId).attr("data-ficheform-role", "label"), formElement.getAttributes(), lang))
                .__escape(formElement.getLabel())
                ._LABEL()
                .SPAN("ficheform-section-Colon")
                .__colon()
                ._SPAN()
                ._DIV();
        hp
                .DIV(HA.attr("data-ficheform-role", "section-area").classes("ficheform-section-Area"))
                .__breakLine()
                .TEXTAREA(HA.name(name).id(textId).cols(77).rows(formElement.getRows()).attr("data-ficheform-role", "sectiontext"))
                .__escape(formElement.getFormattedText(), true)
                ._TEXTAREA()
                .__breakLine()
                ._DIV()
                ._SECTION();
        return true;
    }

    private static String getPrefixedName(CorpusField corpusField, String namePrefix) {
        FieldKey fieldKey = corpusField.getFieldKey();
        return CorpusFieldFormHtml.getPrefixedName(fieldKey.toString(), namePrefix);
    }

    private static String getPrefixedName(FieldKey fieldKey, short subtype, String namePrefix) {
        SubfieldKey subfieldKey = SubfieldKey.toSubfieldKey(fieldKey, subtype);
        return CorpusFieldFormHtml.getPrefixedName(subfieldKey.toString(), namePrefix);
    }

    private static String getPrefixedName(String name, String namePrefix) {
        if ((namePrefix != null) && (namePrefix.length() > 0)) {
            name = namePrefix + "_" + name;
        }
        return name;
    }

    private static String getEntryType(CorpusField corpusField) {
        switch (corpusField.getFicheItemType()) {
            case CorpusField.PERSONNE_FIELD:
                return "redacteur-code";
            case CorpusField.LANGUE_FIELD:
                return "lang-code";
            case CorpusField.PAYS_FIELD:
                return "country-code";
            case CorpusField.DATATION_FIELD:
                if (corpusField.getFieldKey().isProprieteFieldKey()) {
                    return "datation-unique";
                } else {
                    return null;
                }
            default:
                return null;
        }
    }

    private static String getPiocheLimit(CorpusField corpusField) {
        if (corpusField.getFieldKey().isProprieteFieldKey()) {
            return "1";
        } else {
            return "-1";
        }
    }

    private static HtmlAttributes toEntryAttributes(String name, FormElement formElement) {
        return toEntryAttributes(name, formElement, null, "ficheform-standard-Entry");
    }

    private static HtmlAttributes toEntryAttributes(String name, FormElement formElement, String entryType) {
        return toEntryAttributes(name, formElement, entryType, "ficheform-standard-Entry");
    }

    private static HtmlAttributes toEntryAttributes(String name, FormElement formElement, String entryType, String classes) {
        String key = name.replace(':', '_');
        HtmlAttributes htmlAttributes = HA
                .attr("data-ficheform-role", "entry")
                .attr("data-ficheform-key", key)
                .attr(formElement.isMandatory(), "data-ficheform-mandatory", "1")
                .attr((entryType != null), "data-ficheform-entry", entryType)
                .classes(classes);
        CommonFormHtml.addClasses(htmlAttributes, formElement.getAttributes());
        return htmlAttributes;
    }

    private static boolean startSubFieldRow(HtmlPrinter hp, String messageKey, String inputId) {
        hp
                .TR()
                .TD()
                .__doublespace()
                .LABEL_for(inputId)
                .__localize(messageKey)
                ._LABEL()
                ._TD()
                .TD();
        return true;
    }

    private static boolean startCurrencySubFieldRow(HtmlPrinter hp, ExtendedCurrency currency, String currencyId) {
        hp
                .TR()
                .TD()
                .__doublespace()
                .LABEL_for(currencyId)
                .__escape(currency.toString())
                ._LABEL()
                ._TD()
                .TD();
        return true;
    }

    private static boolean endSubfieldRow(HtmlPrinter hp) {
        hp
                ._TD()
                ._TR();
        return true;
    }

    public static boolean printStandardLabel(HtmlPrinter hp, String labelText, String inputId) {
        hp
                .__breakLine()
                .LABEL(HA.forId(inputId).attr("data-ficheform-role", "label").classes("ficheform-standard-Label"))
                .__escape(labelText)
                ._LABEL()
                .__breakLine()
                .SPAN("ficheform-standard-Colon")
                .__escape(':')
                ._SPAN()
                .__breakLine();
        return true;
    }

}
