/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.main;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.MainDomain;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.fichotheque.Subset;
import net.fichotheque.exportation.table.TableExportDescription;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class MainHtmlProducerFactory {

    private MainHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (page) {
            case MainDomain.IFRAMES_PAGE: {
                return new IframesHtmlProducer(checker);
            }
            case MainDomain.MENU_PAGE: {
                return new MenuHtmlProducer(checker);
            }
            case MainDomain.START_PAGE: {
                return new StartHtmlProducer(checker);
            }
            case MainDomain.COLLECTIONS_PAGE: {
                return new CollectionsHtmlProducer(checker);
            }
            case MainDomain.FICHES_PAGE: {
                testFichesReload(bdfServer, bdfUser, requestMap);
                return new FichesHtmlProducer(checker);
            }
            case MainDomain.TABLEDISPLAY_PAGE: {
                TableExportDescription tableExportDescription = null;
                String tableExportName = requestMap.getParameter(InteractionConstants.TABLEEXPORT_PARAMNAME);
                if ((tableExportName != null) && (tableExportName.length() > 0)) {
                    tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
                }
                Subset subset = BdfInstructionUtils.getMandatorySubset(bdfServer.getFichotheque(), requestMap);
                if (requestMap.isTrue(MainDomain.REMEMBER_ACTION_PARAMNAME)) {
                    saveTableDisplayPrefs(bdfServer, bdfUser, tableExportDescription, subset);
                }
                return new TableDisplayHtmlProducer(checker, tableExportDescription, subset);
            }
            default:
                return null;
        }
    }

    public static void testFichesReload(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap) {
        if (requestMap.isTrue(MainDomain.FICHES_RELOAD_PARAMNAME)) {
            bdfServer.updateFicheSelection(bdfUser);
        }
    }

    private static void saveTableDisplayPrefs(BdfServer bdfServer, BdfUser bdfUser, TableExportDescription tableExportDescription, Subset subset) {
        AttributeKey attributeKey = BdfUserSpace.toTableAttributeKey(subset);
        Attribute currentAttribute = bdfUser.getPrefs().getAttributes().getAttribute(attributeKey);
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        if (tableExportDescription == null) {
            if (currentAttribute != null) {
                bdfServerEditor.removeAttribute(bdfUser.getPrefs(), attributeKey);
            }
        } else {
            Attribute newAttribute = AttributeBuilder.toAttribute(attributeKey, tableExportDescription.getName());
            boolean equality = false;
            if (currentAttribute != null) {
                equality = AttributeUtils.isEqual(currentAttribute, newAttribute);
            }
            if (!equality) {
                bdfServerEditor.putAttribute(bdfUser.getPrefs(), newAttribute);
            }
        }
        bdfServerEditor.saveChanges();
    }

}
