/* BdfServer_JsonProducers - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.AdministrationDomain;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.BdfURI;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import java.net.URI;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class AdministrationJsonProducerFactory {

    private AdministrationJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        RequestMap requestMap = parameters.getRequestMap();
        Lang lang = bdfUser.getWorkingLang();
        MessageLocalisation messageLocalisation = bdfServer.getL10nManager().getMessageLocalisation(bdfUser.getLangPreference());
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (json) {
            case AdministrationDomain.RESOURCE_TREE_JSON: {
                bdfServer.getL10nManager().update();
                bdfServer.getTransformationManager().update();
                bdfServer.getJsAnalyser().clearCache();
                jsonProducer.add(new ResourceTreeJsonProperty(bdfServer));
                return jsonProducer;
            }
            case AdministrationDomain.STREAMTEXT_JSON: {
                String path = BdfInstructionUtils.getMandatoryParameter(requestMap, "path");
                URI uri = BdfURI.toAbsoluteBdfURI(path);
                if (uri == null) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue("path", path));
                }
                jsonProducer.add(new StreamTextJsonProperty(bdfServer, uri));
                return jsonProducer;
            }
            case AdministrationDomain.ROLE_ARRAY_JSON: {
                checker.checkGlobalAdmin();
                jsonProducer.add(new RoleArrayJsonProperty(bdfServer, lang));
                return jsonProducer;
            }
            case AdministrationDomain.ROLE_JSON: {
                checker.checkGlobalAdmin();
                Role role = getRole(bdfServer, requestMap, bdfCommandResult);
                jsonProducer.add(new RoleJsonProperty(role, lang));
                return jsonProducer;
            }
            default:
                return null;
        }
    }

    private static Role getRole(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Role role = null;
        if (bdfCommandResult != null) {
            role = (Role) bdfCommandResult.getResultObject(BdfInstructionConstants.ROLE_OBJ);
        }
        if (role == null) {
            role = BdfInstructionUtils.getMandatoryRole(bdfServer.getPermissionManager(), requestMap);
        }
        return role;
    }

}
