/* BdfServer_JsonProducers - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.pioche;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.PiocheDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.externalsource.ExternalSourceUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.RoleUtils;
import java.text.ParseException;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.externalsource.ExternalItem;
import net.fichotheque.externalsource.ExternalSource;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.selection.MotcleQueryParser;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.fichotheque.tools.selection.SelectionEngines;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.CodeCatalog;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public final class PiocheJsonProducerFactory {

    private PiocheJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        Fichotheque fichotheque = bdfServer.getFichotheque();
        Lang lang = bdfUser.getWorkingLang();
        try {
            lang = Lang.parse(requestMap.getParameter("lang"));
        } catch (ParseException pe) {
        }
        PermissionSummary permissionSummary = bdfUser.getPermissionSummary(bdfServer);
        switch (json) {
            case PiocheDomain.FICHE_JSON: {
                return new FichesJsonProducer(bdfServer, getFiches(bdfServer, bdfUser, permissionSummary, requestMap), BdfInstructionUtils.getCellEngine(bdfServer, bdfUser, permissionSummary, requestMap));
            }
            case PiocheDomain.MOTCLE_JSON: {
                return new MotclesJsonProducer(bdfServer, lang, getMotscles(bdfServer, permissionSummary, lang, requestMap), BdfInstructionUtils.getCellEngine(bdfServer, bdfUser, permissionSummary, requestMap));
            }
            case PiocheDomain.EXTERNALITEM_JSON: {
                return new ExternalItemsJsonProducer(getExternalItems(bdfServer, lang, requestMap));
            }
            case PiocheDomain.REDACTEUR_JSON: {
                Condition condition = getCondition(requestMap);
                RedacteursJsonProducer redacteurPiocheJsonProducer = new RedacteursJsonProducer(fichotheque, lang, getSphereSet(requestMap, fichotheque));
                redacteurPiocheJsonProducer.setDefaultSphereKey(getDefaultSphereKey(requestMap));
                redacteurPiocheJsonProducer.init(condition);
                return redacteurPiocheJsonProducer;
            }
            case PiocheDomain.LANGUE_JSON: {
                return new CodesJsonProducer(bdfServer, bdfUser.getLangPreference(), getLangCodeSet(bdfServer, bdfUser, requestMap));
            }
            case PiocheDomain.PAYS_JSON: {
                return new CodesJsonProducer(bdfServer, bdfUser.getLangPreference(), getCountryCodeSet(bdfServer, bdfUser, requestMap));
            }
            case PiocheDomain.FIELD_JSON: {
                Condition condition = getCondition(requestMap);
                FieldsJsonProducer fieldPiocheJsonProducer = new FieldsJsonProducer(bdfServer, bdfUser.getLangPreference(), getFieldType(requestMap));
                fieldPiocheJsonProducer.init(condition);
                return fieldPiocheJsonProducer;
            }
            default:
                return null;
        }
    }

    private static SubsetKey getDefaultSphereKey(RequestMap requestMap) {
        String param = requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME);
        if (param == null) {
            return null;
        }
        try {
            SubsetKey sphereKey = SubsetKey.parse(param);
            if (sphereKey.isSphereSubset()) {
                return sphereKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private static String getFieldType(RequestMap requestMap) {
        String param = requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME);
        if (param == null) {
            return PiocheDomain.ALL_FIELDTYPE;
        } else {
            return param;
        }
    }

    private static Set<Thesaurus> getThesaurusSet(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam == null) {
            return null;
        }
        String[] thesaurusArray = StringUtils.getTechnicalTokens(subsetsParam, true);
        if (thesaurusArray.length == 0) {
            return null;
        }
        Set<Thesaurus> set = new LinkedHashSet<Thesaurus>();
        for (String thesaurusName : thesaurusArray) {
            Thesaurus thesaurus = FichothequeUtils.getThesaurus(fichotheque, thesaurusName);
            if (thesaurus != null) {
                set.add(thesaurus);
            }
        }
        return set;
    }

    private static Set<Corpus> getCorpusSet(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam == null) {
            return null;
        }
        String[] corpusArray = StringUtils.getTechnicalTokens(subsetsParam, true);
        if (corpusArray.length == 0) {
            return null;
        }
        Set<Corpus> set = new LinkedHashSet<Corpus>();
        for (String corpusName : corpusArray) {
            Corpus corpus = FichothequeUtils.getCorpus(fichotheque, corpusName);
            if (corpus != null) {
                set.add(corpus);
            }
        }
        return set;
    }

    private static Set<Sphere> getSphereSet(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam == null) {
            return null;
        }
        String[] sphereArray = StringUtils.getTechnicalTokens(subsetsParam, true);
        if (sphereArray.length == 0) {
            return null;
        }
        Set<Sphere> set = new LinkedHashSet<Sphere>();
        for (String sphereName : sphereArray) {
            Sphere sphere = FichothequeUtils.getSphere(fichotheque, sphereName);
            if (sphere != null) {
                set.add(sphere);
            }
        }
        if (set.isEmpty()) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
        }
        return set;
    }

    private static SubsetKey getCroisementSubsetKey(RequestMap requestMap) throws BdfInstructionException {
        String subsetString = requestMap.getParameter(PiocheDomain.CROISEMENT_PARAMNAME);
        if (subsetString == null) {
            return null;
        }
        subsetString = subsetString.trim();
        if (subsetString.length() == 0) {
            return null;
        }
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetString);
            return subsetKey;
        } catch (ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.CROISEMENT_PARAMNAME, subsetString));
        }
    }

    private static Condition getCondition(RequestMap requestMap) throws BdfInstructionException {
        String q = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.Q_PARAMNAME);
        return ConditionsUtils.parseSimpleCondition(q);
    }

    private static Collection<Motcle> getMotscles(BdfServer bdfServer, PermissionSummary permissionSummary, Lang lang, RequestMap requestMap) throws BdfInstructionException {
        Set<Thesaurus> thesaurusSet = getThesaurusSet(requestMap, bdfServer.getFichotheque());
        SubsetKey shouldNotCroisementSubsetKey = getCroisementSubsetKey(requestMap);
        if ((thesaurusSet != null) && (thesaurusSet.size() == 1)) {
            Thesaurus thesaurus = thesaurusSet.iterator().next();
            lang = BdfServerUtils.checkLangDisponibility(bdfServer, thesaurus, lang);
        }
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, lang)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toSelectionContext();
        String q = requestMap.getParameter(PiocheDomain.Q_PARAMNAME);
        if (q != null) {
            Condition condition = ConditionsUtils.parseSimpleCondition(q);
            if (condition != null) {
                return SelectionEngines.run(selectionContext, thesaurusSet, shouldNotCroisementSubsetKey, condition);
            } else {
                return SelectionEngines.run(selectionContext, thesaurusSet, shouldNotCroisementSubsetKey);
            }
        } else {
            String xml = requestMap.getParameter(PiocheDomain.XML_PARAMNAME);
            if (xml != null) {
                Document document;
                try {
                    document = DOMUtils.parseDocument(xml);
                } catch (SAXException sae) {
                    throw new BdfInstructionException(LogUtils.error("_ error.exception.xml.sax", sae.getMessage()));
                }
                Element root = document.getDocumentElement();
                switch (root.getTagName()) {
                    case "motcle-query":
                    case "motcle-select":
                        MotcleQuery motcleQuery = SelectionDOMUtils.getMotcleQuery(bdfServer.getFichotheque(), root);
                        return SelectionEngines.run(selectionContext, thesaurusSet, shouldNotCroisementSubsetKey, motcleQuery);
                    default:
                        throw new BdfInstructionException(LogUtils.error("_ error.wrong.xml.root", root.getTagName(), "motcle-query"));
                }
            } else {
                return SelectionEngines.run(selectionContext, thesaurusSet, shouldNotCroisementSubsetKey);
            }
        }
    }

    private static Collection<ExternalItem> getExternalItems(BdfServer bdfServer, Lang lang, RequestMap requestMap) throws BdfInstructionException {
        String subsetsParam = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.SUBSETS_PARAMNAME);
        Set<Thesaurus> thesaurusSet = getThesaurusSet(requestMap, bdfServer.getFichotheque());
        if (thesaurusSet == null) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
        }
        Thesaurus thesaurus = thesaurusSet.iterator().next();
        ExternalSource externalSource = ExternalSourceUtils.getExternalSource(bdfServer, thesaurus);
        if (externalSource == null) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
        }
        lang = BdfServerUtils.checkLangDisponibility(bdfServer, thesaurus, lang);
        String q = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.Q_PARAMNAME);
        if (q.isEmpty()) {
            return ExternalSourceUtils.EXTERNALITEM_EMPTYLIST;
        }
        return externalSource.search(q, lang);
    }

    private static Collection<FicheMeta> getFiches(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap) throws BdfInstructionException {
        Lang lang = bdfUser.getWorkingLang();
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, lang)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .setFichePredicate(RoleUtils.getFichePredicate(bdfServer.getPermissionManager(), bdfUser.getRedacteur()))
                .toSelectionContext();
        Set<Corpus> corpusSet = getCorpusSet(requestMap, bdfServer.getFichotheque());
        String q = requestMap.getParameter(PiocheDomain.Q_PARAMNAME);
        if (q != null) {
            Condition condition = ConditionsUtils.parseSimpleCondition(q);
            MotcleQuery motcleQuery = null;
            String motcleQueryString = requestMap.getParameter(PiocheDomain.MOTCLESELECTOPTIONS_PARAMNAME);
            if (motcleQueryString != null) {
                motcleQuery = MotcleQueryParser.parse(motcleQueryString);
            }
            return SelectionEngines.run(selectionContext, corpusSet, condition, motcleQuery);
        } else {
            String xml = requestMap.getParameter(PiocheDomain.XML_PARAMNAME);
            if (xml != null) {
                Document document;
                try {
                    document = DOMUtils.parseDocument(xml);
                } catch (SAXException sae) {
                    throw new BdfInstructionException(LogUtils.error("_ error.exception.xml.sax", sae.getMessage()));
                }
                Element root = document.getDocumentElement();
                switch (root.getTagName()) {
                    case "fiche-query":
                    case "fiche-select":
                        FicheQuery ficheQuery = SelectionDOMUtils.getFicheQuery(bdfServer.getFichotheque(), root);
                        return SelectionEngines.run(selectionContext, corpusSet, ficheQuery);
                    default:
                        throw new BdfInstructionException(LogUtils.error("_ error.wrong.xml.root", root.getTagName(), "fiche-query"));
                }
            } else {
                return SelectionEngines.run(selectionContext, corpusSet);
            }
        }
    }

    private static Set<String> getCountryCodeSet(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap) throws BdfInstructionException {
        Condition condition = getCondition(requestMap);
        CodeCatalog codeCatalog = bdfServer.getL10nManager().getCodeCatalog();
        if (condition != null) {
            return codeCatalog.getCountryCodeSet(condition, bdfUser.getLangPreference());
        } else {
            return codeCatalog.getCountryCodeSet();
        }
    }

    private static Set<String> getLangCodeSet(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap) throws BdfInstructionException {
        Condition condition = getCondition(requestMap);
        CodeCatalog codeCatalog = bdfServer.getL10nManager().getCodeCatalog();
        if (condition != null) {
            return codeCatalog.getLangCodeSet(condition, bdfUser.getLangPreference());
        } else {
            return codeCatalog.getLangCodeSet();
        }
    }

}
