/* FichothequeLib_Tools - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.LiaisonSortKey;
import net.fichotheque.croisement.Lien;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.run.AddendaExtractor;
import net.fichotheque.extraction.run.DocumentExtractInfo;
import net.fichotheque.extraction.run.ExtraitAddendaResult;
import net.fichotheque.utils.CroisementUtils;


/**
 *
 * @author Vincent Calame
 */
class AddendaExtractorImpl implements AddendaExtractor {

    private final Map<SubsetKey, ResultBuilder> builderMap = new TreeMap<SubsetKey, ResultBuilder>();
    private final ExtraitAddendaDef extraitAddendaDef;

    AddendaExtractorImpl(ExtraitAddendaDef extraitAddendaDef) {
        this.extraitAddendaDef = extraitAddendaDef;
    }

    @Override
    public void add(Document document, Croisement croisement) {
        Addenda addenda = document.getAddenda();
        ResultBuilder resultBuilder = builderMap.get(addenda.getSubsetKey());
        DocumentFilter documentFilter = extraitAddendaDef.getDocumentFilter();
        if (resultBuilder == null) {
            resultBuilder = new ResultBuilder(addenda);
            builderMap.put(addenda.getSubsetKey(), resultBuilder);
        }
        resultBuilder.add(document, documentFilter, croisement);
    }

    @Override
    public boolean containsDocument() {
        return (!builderMap.isEmpty());
    }

    @Override
    public ExtraitAddendaResult getExtraitAddendaResult() {
        int size = builderMap.size();
        Addenda[] addendaArray = new Addenda[size];
        List<DocumentExtractInfo>[] documentExtractInfoListArray = new List[size];
        int p = 0;
        for (ResultBuilder resultBuilder : builderMap.values()) {
            addendaArray[p] = resultBuilder.addenda;
            documentExtractInfoListArray[p] = resultBuilder.getDocumentExtractInfoList();
            p++;
        }
        InternalExtraitAddendaResult extraitAddendaResult = new InternalExtraitAddendaResult(extraitAddendaDef, addendaArray, documentExtractInfoListArray);
        return extraitAddendaResult;
    }


    private class ResultBuilder {

        private Addenda addenda;
        private SortedMap<LiaisonSortKey, InternalDocumentExtractInfo> map = new TreeMap<LiaisonSortKey, InternalDocumentExtractInfo>();

        private ResultBuilder(Addenda addenda) {
            this.addenda = addenda;
        }

        private void add(Document document, DocumentFilter documentFilter, Croisement croisement) {
            int position;
            if (croisement != null) {
                Lien lien = croisement.getLienList().get(0);
                position = CroisementUtils.getPosition(document, croisement.getCroisementKey(), lien);
            } else {
                position = map.size() + 1;
            }
            LiaisonSortKey key = new LiaisonSortKey(document.getId(), position);
            map.put(key, new InternalDocumentExtractInfo(document, documentFilter, croisement));
        }

        private List<DocumentExtractInfo> getDocumentExtractInfoList() {
            return new ArrayList(map.values());
        }

    }


    private static class InternalExtraitAddendaResult implements ExtraitAddendaResult {

        private final Addenda[] addendaArray;
        private final List<DocumentExtractInfo>[] documentExtractInfoListArray;
        private final ExtraitAddendaDef extraitAddendaDef;

        private InternalExtraitAddendaResult(ExtraitAddendaDef extraitAddendaDef, Addenda[] addendaArray, List<DocumentExtractInfo>[] documentExtractInfoListArray) {
            this.extraitAddendaDef = extraitAddendaDef;
            this.addendaArray = addendaArray;
            this.documentExtractInfoListArray = documentExtractInfoListArray;
        }

        @Override
        public ExtraitAddendaDef getExtraitAddendaDef() {
            return extraitAddendaDef;
        }

        @Override
        public int getAddendaCount() {
            return addendaArray.length;
        }

        @Override
        public Addenda getAddenda(int index) {
            return addendaArray[index];
        }

        @Override
        public List<DocumentExtractInfo> getDocumentExtractInfoList(int index) {
            return documentExtractInfoListArray[index];
        }

    }


    private static class InternalDocumentExtractInfo implements DocumentExtractInfo {

        private final Document document;
        private final DocumentFilter documentFilter;
        private final Croisement croisement;

        public InternalDocumentExtractInfo(Document document, DocumentFilter documentFilter, Croisement croisement) {
            this.document = document;
            this.documentFilter = documentFilter;
            this.croisement = croisement;
        }

        @Override
        public Document getDocument() {
            return document;
        }

        @Override
        public DocumentFilter getDocumentFilter() {
            return documentFilter;
        }

        @Override
        public Croisement getCroisement() {
            return croisement;
        }

    }

}
