/* FichothequeLib_Tools - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.SubsetKey;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.LiaisonSortKey;
import net.fichotheque.croisement.Lien;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.run.ExtraitThesaurusResult;
import net.fichotheque.extraction.run.MotcleExtractInfo;
import net.fichotheque.extraction.run.ThesaurusExtractor;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.CroisementUtils;


/**
 *
 * @author Vincent Calame
 */
class ThesaurusExtractorImpl implements ThesaurusExtractor {

    private final Map<SubsetKey, ResultBuilder> builderMap = new TreeMap<SubsetKey, ResultBuilder>();
    private final ExtraitThesaurusDef extraitThesaurusDef;

    ThesaurusExtractorImpl(ExtraitThesaurusDef extraitThesaurusDef) {
        this.extraitThesaurusDef = extraitThesaurusDef;
    }

    @Override
    public ExtraitThesaurusDef getExtraitThesaurusDef() {
        return extraitThesaurusDef;
    }

    @Override
    public void add(Motcle motcle, Croisement croisement) {
        Thesaurus thesaurus = motcle.getThesaurus();
        MotcleFilter motcleFilter = extraitThesaurusDef.getMotcleFilter();
        ResultBuilder resultBuilder = builderMap.get(thesaurus.getSubsetKey());
        if (resultBuilder == null) {
            resultBuilder = new ResultBuilder(thesaurus);
            builderMap.put(thesaurus.getSubsetKey(), resultBuilder);
        }
        resultBuilder.add(motcle, motcleFilter, croisement);

    }

    @Override
    public boolean containsMotcle() {
        return (!builderMap.isEmpty());
    }

    @Override
    public ExtraitThesaurusResult getExtraitThesaurusResult() {
        int size = builderMap.size();
        Thesaurus[] thesaurusArray = new Thesaurus[size];
        List<MotcleExtractInfo>[] motcleExtractInfoListArray = new List[size];
        int p = 0;
        for (ResultBuilder resultBuilder : builderMap.values()) {
            thesaurusArray[p] = resultBuilder.thesaurus;
            motcleExtractInfoListArray[p] = resultBuilder.getMotcleExtractInfoList();
            p++;
        }
        InternalExtraitThesaurusResult extraitThesaurusResult = new InternalExtraitThesaurusResult(extraitThesaurusDef, thesaurusArray, motcleExtractInfoListArray);
        return extraitThesaurusResult;
    }


    private static class ResultBuilder {

        private final Thesaurus thesaurus;
        private final SortedMap<LiaisonSortKey, InternalMotcleExtractInfo> map = new TreeMap<LiaisonSortKey, InternalMotcleExtractInfo>();

        private ResultBuilder(Thesaurus thesaurus) {
            this.thesaurus = thesaurus;
        }

        private void add(Motcle motcle, MotcleFilter motcleFilter, Croisement croisement) {
            int position;
            if (croisement != null) {
                Lien lien = croisement.getLienList().get(0);
                position = CroisementUtils.getPosition(motcle, croisement.getCroisementKey(), lien);
            } else {
                position = map.size() + 1;
            }
            LiaisonSortKey key = new LiaisonSortKey(motcle.getId(), position);
            map.put(key, new InternalMotcleExtractInfo(motcle, motcleFilter, croisement));
        }

        private List<MotcleExtractInfo> getMotcleExtractInfoList() {
            return new ArrayList(map.values());
        }

    }


    private static class InternalExtraitThesaurusResult implements ExtraitThesaurusResult {

        private final Thesaurus[] thesaurusArray;
        private final List<MotcleExtractInfo>[] motcleExtractInfoListArray;
        private final ExtraitThesaurusDef extraitThesaurusDef;

        private InternalExtraitThesaurusResult(ExtraitThesaurusDef extraitThesaurusDef, Thesaurus[] thesaurusArray, List<MotcleExtractInfo>[] motcleExtractInfoListArray) {
            this.extraitThesaurusDef = extraitThesaurusDef;
            this.thesaurusArray = thesaurusArray;
            this.motcleExtractInfoListArray = motcleExtractInfoListArray;
        }

        @Override
        public ExtraitThesaurusDef getExtraitThesaurusDef() {
            return extraitThesaurusDef;
        }

        @Override
        public int getThesaurusCount() {
            return thesaurusArray.length;
        }

        @Override
        public Thesaurus getThesaurus(int index) {
            return thesaurusArray[index];
        }

        @Override
        public List<MotcleExtractInfo> getMotcleExtractInfoList(int index) {
            return motcleExtractInfoListArray[index];
        }

    }


    private static class InternalMotcleExtractInfo implements MotcleExtractInfo {

        private final Motcle motcle;
        private final MotcleFilter motcleFilter;
        private final Croisement croisement;

        private InternalMotcleExtractInfo(Motcle motcle, MotcleFilter motcleFilter, Croisement croisement) {
            this.motcle = motcle;
            this.motcleFilter = motcleFilter;
            this.croisement = croisement;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

        @Override
        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        @Override
        public Croisement getCroisement() {
            return croisement;
        }

    }

}
