/* FichothequeLib_Xml - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.storage;

import java.io.IOException;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptions;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.FieldOptionsUtils;
import net.fichotheque.xml.FichothequeXMLUtils;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.money.Currencies;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CorpusMetadataXMLPart extends XMLPart {

    public CorpusMetadataXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendCorpusMetadata(CorpusMetadata corpusMetadata) throws IOException {
        openTag("corpus-metadata");
        FichothequeXMLUtils.appendPhrases(corpusMetadata, this);
        appendFields(corpusMetadata);
        appendFieldGeneration(corpusMetadata);
        appendOptions(corpusMetadata);
        appendL10n(corpusMetadata);
        AttributeUtils.addAttributes(this, corpusMetadata.getAttributes());
        closeTag("corpus-metadata");
    }

    private void appendFields(CorpusMetadata corpusMetadata) throws IOException {
        openTag("fields");
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            addEmptyElement("soustitre");
        }
        for (CorpusField corpusField : corpusMetadata.getProprieteList()) {
            startOpenTag("propriete");
            addAttribute("name", corpusField.getFieldKey().getFieldName());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();

        }
        for (CorpusField corpusField : corpusMetadata.getInformationList()) {
            startOpenTag("information");
            addAttribute("name", corpusField.getFieldKey().getFieldName());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();
        }
        for (CorpusField corpusField : corpusMetadata.getSectionList()) {
            startOpenTag("section");
            addAttribute("name", corpusField.getFieldKey().getFieldName());
            closeEmptyTag();
        }
        closeTag("fields");
    }

    private void appendFicheItemAttribute(CorpusField corpusField) throws IOException {
        addAttribute("fiche-item", CorpusMetadataUtils.ficheItemTypeToString(corpusField.getFicheItemType()));
    }

    private void appendL10n(CorpusMetadata corpusMetadata) throws IOException {
        openTag("l10n");
        List<CorpusField> list = CorpusMetadataUtils.getCorpusFieldList(corpusMetadata);
        for (CorpusField corpusField : list) {
            startOpenTag("field");
            addAttribute("field-key", corpusField.getFieldKey().toString());
            endOpenTag();
            LabelUtils.addLabels(this, corpusField.getLabels());
            closeTag("field");
        }
        closeTag("l10n");
    }

    private void appendFieldGeneration(CorpusMetadata corpusMetadata) throws IOException {
        String rawString = corpusMetadata.getFieldGeneration().getRawString();
        if (!rawString.isEmpty()) {
            openTag("field-generation");
            addCData(rawString);
            closeTag("field-generation", false);
        }
    }

    private void appendOptions(CorpusMetadata corpusMetadata) throws IOException {
        openTag("options");
        CorpusField geolocalisationField = corpusMetadata.getGeolocalisationField();
        if (geolocalisationField != null) {
            String value = null;
            appendOption(FieldOptionsConstants.GEOLOCALISATIONFIELD_OPTION, value, geolocalisationField.getFieldKey());
        }
        appendOptions(corpusMetadata.getCorpusField(FieldKey.ID));
        appendOptions(corpusMetadata.getCorpusField(FieldKey.TITRE));
        appendOptions(corpusMetadata.getCorpusField(FieldKey.LANG));
        appendOptions(corpusMetadata.getCorpusField(FieldKey.REDACTEURS));
        CorpusField soustitre = corpusMetadata.getCorpusField(FieldKey.SOUSTITRE);
        if (soustitre != null) {
            appendOptions(soustitre);
        }
        for (CorpusField corpusField : corpusMetadata.getProprieteList()) {
            appendOptions(corpusField);
        }
        for (CorpusField corpusField : corpusMetadata.getInformationList()) {
            appendOptions(corpusField);
        }
        for (CorpusField corpusField : corpusMetadata.getSectionList()) {
            appendOptions(corpusField);
        }
        closeTag("options");
    }

    private void appendOptions(CorpusField corpusField) throws IOException {
        FieldOptions fieldOptions = corpusField.getFieldOptions();
        FieldKey fieldKey = corpusField.getFieldKey();
        Short informationDisplay = fieldOptions.getInformationDisplay();
        if (informationDisplay != null) {
            if (informationDisplay != FieldOptionsUtils.getDefaultInformationDisplay()) {
                appendOption(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, FieldOptionsUtils.informationDisplayToString(informationDisplay), fieldKey);
            }
        }
        SubsetKey sphereKey = fieldOptions.getDefaultSphereKey();
        if (sphereKey != null) {
            appendOption(FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION, sphereKey.toString(), fieldKey);
        }
        boolean bool = fieldOptions.getSubfieldDisplay();
        if (bool) {
            appendOption(FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, "1", fieldKey);
        }
        Currencies currencies = fieldOptions.getCurrencies();
        if (currencies != null) {
            int length = currencies.size();
            String[] currencyArray = new String[length];
            for (int i = 0; i < length; i++) {
                currencyArray[i] = currencies.get(i).getCurrencyCode();
            }
            appendOption(FieldOptionsConstants.CURRENCYARRAY_OPTION, currencyArray, fieldKey);
        }
        List<CorpusField> corpusFieldList = fieldOptions.getAddressFieldList();
        if (corpusFieldList != null) {
            int length = corpusFieldList.size();
            String[] fieldArray = new String[length];
            for (int i = 0; i < length; i++) {
                fieldArray[i] = corpusFieldList.get(i).getFieldKey().toString();
            }
            appendOption(FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION, fieldArray, fieldKey);
        }
        Short langScope = fieldOptions.getLangScope();
        if (langScope != null) {
            if (langScope != FieldOptionsUtils.getDefaultLangScope(fieldKey)) {
                appendOption(FieldOptionsConstants.LANGSCOPE_OPTION, FieldOptionsUtils.langScopeToString(langScope), fieldKey);
            }
        }
        Langs langs = fieldOptions.getLangs();
        if (langs != null) {
            int length = langs.size();
            String[] langArray = new String[length];
            for (int i = 0; i < length; i++) {
                langArray[i] = langs.get(i).toString();
            }
            appendOption(FieldOptionsConstants.LANGARRAY_OPTION, langArray, fieldKey);
        }
        String[] stringOptionNameArray = corpusField.getStringOptionNameArray();
        int length = stringOptionNameArray.length;
        for (int i = 0; i < length; i++) {
            appendStringOption(corpusField, stringOptionNameArray[i]);
        }
    }

    private void appendStringOption(CorpusField corpusField, String optionName) throws IOException {
        String optionValue = corpusField.getStringOption(optionName);
        if (optionValue != null) {
            appendOption(optionName, optionValue, corpusField.getFieldKey());
        }
    }

    private void appendOption(String name, String value, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("field-key", fieldKey.toString());
        addAttribute("value", value);
        closeEmptyTag();
    }

    private void appendOption(String name, String[] values, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("field-key", fieldKey.toString());
        endOpenTag();
        int length = values.length;
        for (int i = 0; i < length; i++) {
            String value = values[i];
            if ((value != null) && (value.length() > 0)) {
                addSimpleElement("value", value);
            } else {
                addEmptyElement("value");
            }
        }
        closeTag("option");
    }

}
